/**************************************************
opengate mac addr auth program

 cgi module for reg/update by owner oneself

Copyright (C) 2011 Opengate Project Team
Written by Yoshiaki Watanabe

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Email: watanaby@is.saga-u.ac.jp
**************************************************/

#include	"opengatemmng.h"

/***************************************************/
/*  main routine called as cgi from Web server     */
/***************************************************/
int  main(int argc, char **argv)
{
  char language[WORDMAXLN]="";
  char userId[USERMAXLN]="";
  char extraId[USERMAXLN]="";
  char requestStr[BUFFMAXLN]=""; /* http request string */
  char* progName="";
  char mailDefault[BUFFMAXLN]="";
  char macAddress[ADDRMAXLN]="";
  char macAddressReq[ADDRMAXLN]="";
  char ipAddress[ADDRMAXLN]="";
  char* dev; /* nic interface name */
  char ipAddressInQueue[ADDRMAXLN]="";
  char macAddressInQueue[ADDRMAXLN]="";
  int registered=FALSE; /* the device is registered or not */
  char deviceName[WORDMAXLN]=""; /* client device name */
  char mailAddress[BUFFMAXLN]=""; /* mail address to receive warning mail */
  int isNatOrRouter=0; /* flag for nat or router insertion (checked in md) */
  char redirectedUrl[BUFFMAXLN]=""; /* url from which the page is forwarded */
  int modified=FALSE; /* register or update is done */

  /* drop root privilege */
  seteuid(getuid());

  /* if this is executed in shell with '-v' option, show makedir */  
  if(argc>1 && strcmp(argv[1],"-v")==0){
    printf("makedir: %s\n", MAKEDIR);
    exit(0);
  }

  /* save program load path */
  saveLoadPath(argv[0]);
  progName = getProgramName();

  /* prepare config file */
  if(OpenConfFile()==-1){
    PutMessageToClient("Check config file by running this cgi on console");
    return 0;
  }
 
  /* start log */
  errToSyslog(atoi(GetConfValue("Syslog/Enable")));
  openlog(progName, LOG_PID, atoi(GetConfValue("Syslog/Facility")));

  /* initialize config */
  InitConf();
  if(!InitMngDb()) return 0;
  if(!InitWorkDb()) return 0;

  /* get language and other from query string */
  GetLangFromQueryString(language);
  GetRedirectedUrlFromQueryString(redirectedUrl);

  /* check client's http-agent, if it is not defined in conf, exit */
  if(!IsAccessedFromAllowableAgent()){
    SetMessage(UnallowableAgent);
    PutDenyToClient(language);
    CloseMngDb();
    return 0;
  }

  /* get post data */
  GetPostData(requestStr, BUFFMAXLN);

  /* get userid. if not get, show auth page and exit */
  if(isNull(GetConfValue("OwnCgi"))){
        err_msg("ERR at %s#%d: OwnCgi cannot get from conf file",__FILE__,__LINE__);
	return 0;
  }
  if(!GetUserId(requestStr, userId, extraId, language, NORMALUSER,
		GetConfValue("OwnCgi"),mailDefault,redirectedUrl)){
    CloseMngDb();
    return 0;
  }

  /* if many modify request per day, ignore the request */
  if(CountMacModifyPerDayInMngDb(userId,extraId) 
     > atoi(GetConfValue("MaxMacModifyPerDay"))){
    SetMessage(ModifyCountOver);
    PutDenyToClient(language);
    CloseMngDb();
    return 0;
  }

  /* get client ip address */
  strncpy(ipAddress, getenv("REMOTE_ADDR"), ADDRMAXLN);

  /* get mac addresses from arp/ndp */
  if(isNull(dev=GetConfValue("Device"))){
        err_msg("ERR at %s#%d: Device cannot get from conf file",__FILE__,__LINE__);
	return 0;
  }

  Initqueue();
  if(strchr(ipAddress,':')==NULL) GetMacAddrListFromArp(dev);
  else GetMacAddrListFromNdp(dev);

  /* search mac address corresponding to client ip address */
  while(Dequeue(macAddressInQueue, ipAddressInQueue)){
    if(strcmp(ipAddress, ipAddressInQueue)==0){
      strncpy(macAddress, macAddressInQueue, ADDRMAXLN);
      break;
    }
  }
  Freequeue();

  /* if accessed via nat or router, return  */
  /* if nat, func returns 1. if router, func returns 2. */
  isNatOrRouter=IsNatSuspectedInWorkDb(macAddress);
  if(isNatOrRouter==NAT || isNatOrRouter==ROUTER){
    SetMessage(SuspectNat);
    PutDenyToClient(language);
    CloseMngDb();
    return 0;
  }

  /* is the mac address registered */
  registered=IsMacAddrFoundInMngDb(macAddress);

  /* analyze request from client */
  if(!isNull(requestStr)){
    
    /* if mac is already registered, execute update requests */
    if(registered){
      if(AnalyzeUpdateRequestAndExecute(requestStr, userId, extraId)){
	SetMessage(UpdateSuccess);
	modified=TRUE;
      }
    }

    /* else execute register requests */
    else{
      if(AnalyzeRegisterRequest(requestStr, macAddressReq, deviceName, mailAddress)){

	/* register to db */
	RegistMacAddrToMngDb(macAddress,deviceName,userId,extraId,mailAddress);
	PutMacModifyLogToMngDb(userId, extraId, macAddress, 'R');
	PutMacAddressToServers(macAddress);
	SetMessage(RegisterSuccess);
	modified=TRUE;
	registered=TRUE;
      }
    }
  }

  /* if registration/Updating is done, goto previous site */
  if(modified && !isNull(redirectedUrl)){
    ReturnToRedirectedPage(redirectedUrl, language);
  }

  /* if registered, prepare update response and send to client */
  else if(registered){

    /* last arg=1 means that the page is controlled by owner */
    PutUpdatePageToClient(language, userId, extraId, 1, redirectedUrl);
  }

  /* else prepare register page and send */
  else{

    /* restore mail address */
    if(isNull(mailAddress)) strncpy(mailAddress, mailDefault, BUFFMAXLN);

    /* send register page to the client */
    /* last arg=1 means that the page is controlled by owner */
    PutRegisterPageToClient(language, macAddress, deviceName, mailAddress, userId, extraId, 1, redirectedUrl);
  }

  /* finalize */
  CloseMngDb();
  return 0;
}
